-- ADTS decoding, inside TCP/UDP

-- http://www.mp3-tech.org/programmer/frame_header.html
-- http://wiki.multimedia.cx/index.php?title=ADTS

-- requires liblua bitop http://bitop.luajit.org
-- included in Wireshark now
-- local bit = require("bit")
local function bext(value, bitoffset, bitlen)
  return bit.band( bit.rshift(value, bitoffset) , bit.lshift(1,bitlen)-1)
end
local function swap16(value)
  return bit.rshift( bit.bswap( value ), 16)
end
local band = bit.band


-- declare our protocol
adts_proto = Proto("ADTS","ADTS Protocol")

-- Protocol Fields strings
local mpeg = { [0]="MPEG-4", "MPEG-2" }
local profiles = { [0]="AAC-Main", "AAC-LC", "AAC-SSR", "AAC-LTP", "SBR", "AAC Scalable"
    -- TODO: fill up whole table !
    , [16]="ER (Error Resilient) AAC LC "
    , [28]="PS" }
local samplerate = { [0]="96", "88.2", "64", "48", "44.1", "32", "24", "22.05", "16", "12", "11.025", "8", "7.35", "reserved", "reserved", "explicit (forbidden)" }
local channels = { [0]="inband", "1", "2", "3", "4", "5", "6", "8" }

-- Protocol Fields definition
local f = adts_proto.fields
f.header = ProtoField.uint64("adts.header"   , "Flags"  , base.HEX, nil, 0xFFFFFFFFFFFFFF00)
    f.sync = ProtoField.uint16("adts.header.sync"      , "Sync"   , base.HEX, nil, 0xFFF0)
    f.mpeg = ProtoField.uint8("adts.header.mpeg"      , "MPEG"   , base.HEX, mpeg, 0x08)
    f.layer = ProtoField.uint8("adts.header.layer"      , "Layer"   , base.DEC, nil, 0x06)
    f.nocrc = ProtoField.uint8("adts.header.nocrc"      , "No CRC?"   , base.DEC, nil, 0x01)
    f.profile = ProtoField.uint8("adts.header.profile"      , "Profile"   , base.DEC, profiles, 0xc0)
    f.samplerate = ProtoField.uint8("adts.header.samplerate"      , "samplerate"   , base.DEC, samplerate, 0x3c)
    f.private = ProtoField.uint8("adts.header.private"      , "private"   , base.DEC, nil, 0x02)
    f.channels = ProtoField.uint16("adts.header.channels"      , "channels"   , base.DEC, channels, 0x01C0)
    f.orig = ProtoField.uint8("adts.header.orig"      , "orig"   , base.DEC, nil, 0x20)
    f.home = ProtoField.uint8("adts.header.home"      , "home"   , base.DEC, nil, 0x10)
    f.copyright = ProtoField.uint8("adts.header.copyright"      , "copyright"   , base.DEC, nil, 0x08)
    f.copystart = ProtoField.uint8("adts.header.copystart"      , "copyright start"   , base.DEC, nil, 0x04)
    f.framelen = ProtoField.uint24("adts.header.framelen"      , "frame length"   , base.DEC, nil, 0x03FFE0)
    f.bufferfullness = ProtoField.uint16("adts.header.bufferfullness"      , "buffer fullness"   , base.DEC, nil, 0x1FFC)
    f.nbframe = ProtoField.uint8("adts.header.nbframe"      , "nb frame -1"   , base.DEC, nil, 0x03)
    f.crc = ProtoField.uint16("adts.header.crc"      , "CRC"   , base.HEX, nil, 0xFFFF)

f.mydata = ProtoField.bytes ("adts.rawdata", "Raw Data")


-- create a function to dissect it
function adts_proto.dissector(buffer,pinfo,tree)
    local header
    local subtree
    local subtree2
    local subtree3
    local infotxt
    local MpegLayer     -- for decoding, common to all headers
    local MpegData
    
    -- MP3 is 4 bytes
    -- ADTS is 7-9 bytes

    if (buffer:len()<4) then
        -- pinfo.cols.protocol = "ADTS Audio: Bad size, no header !"    
        return 0
    else
        local syncword=buffer(0,2):uint()
        -- Is it MP3 ?
        
        if ( (band(syncword,0xFFE0) == 0xFFE0) and (band(syncword,0x0018) ~= 0x0008) and (band(syncword,0x0006) ~= 0x00000) ) then
            -- Yes, MP3 header
            pinfo.cols.protocol = "MP3 Audio (not yet handled)"

            -- TODO: MP3 header !

            
        elseif ( (buffer:len()>=7) and (band(syncword,0xFFF0) == 0xFFF0) and (band(syncword,0x0006) == 0x00000) ) then
            -- Yes, ADTS header
            -- Is There a CRC ?
            local headersize
            local header
            local sync
            local hascrc
            local layer
            local profile
            local profilename

            headersize=7
            hascrc=false
            if ( band(syncword,1)==0) then
                headersize=headersize+2
                hascrc=true
            end
            MpegData=buffer(headersize,buffer:len()-headersize)
            pinfo.cols.protocol = "ADTS Audio"
            layer=bext(buffer(1,1):uint(),3,1)
            profile=bext(buffer(2,1):uint(),6,2)
            if profiles[profile]==nil then
                profilename="?"..profile.."?"
            else
                profilename=profiles[profile]
            end
            if layer==0 then
                MpegLayer=4
            else
                MpegLayer=2
            end
            infotxt= "ADTS Audio (" .. mpeg[layer] .. " " .. profilename .. ")"
            subtree = tree:add(adts_proto,buffer(),"ADTS Audio Chunk")
                subtree2 = subtree:add(buffer(0,headersize),"Header")            
                    -- byte 0-1
                    subtree2:add(f.sync,buffer(0,2))
                    subtree2:add(f.mpeg,buffer(1,1))
                    subtree2:add(f.layer,buffer(1,1))
                    subtree2:add(f.nocrc,buffer(1,1))
                    -- byte 2
                    subtree2:add(f.profile,buffer(2,1))
                    subtree2:add(f.samplerate,buffer(2,1))
                    subtree2:add(f.private,buffer(2,1))
                    subtree2:add(f.channels,buffer(2,2))
                    -- byte 3
                    subtree2:add(f.orig,buffer(3,1))
                    subtree2:add(f.home,buffer(3,1))
                    subtree2:add(f.copyright,buffer(3,1))
                    subtree2:add(f.copystart,buffer(3,1))
                    subtree2:add(f.copystart,buffer(3,1))
                    -- byte 4&5
                    subtree2:add(f.framelen,buffer(3,3))
                    -- byte 6
                    subtree2:add(f.bufferfullness,buffer(5,2))
                    subtree2:add(f.nbframe,buffer(6,1))
                    -- byte 7-8
                    if (hascrc) then
                        subtree2:add(f.crc,buffer(7,2))
                    end
        else
            -- pinfo.cols.protocol = "ADTS Audio: Unknown header !"    
            return 0
        end
    end
    if MpegLayer==4 then
        subtree2 = subtree:add(f.mydata, MpegData,"MPEG-4 Data")
        -- Size 32
        local size
        local kind
        local box
        local boxlen
        local boxheader
        
        size=MpegData(0,4):uint()
        kind=MpegData(4,4):uint()
        boxlen=8
        if size==1 then
            -- large size
            size= MpegData(8,8):uint()
            boxlen=boxlen+8
        elseif size==0 then
            -- to the end
            size= MpegData:len()
        else
            -- OK, normal len
        end
        if boxlen>MpegData:len() then
            infotxt = infotxt .. "BAD LEN"
        end
        boxheader=MpegData(0, boxlen)
        
        
        
        -- Type 32
    elseif MpegLayer==2 then
        subtree2 = subtree:add(f.mydata, MpegData,"MPEG-2 Data")
    elseif MpegLayer==1 then
        subtree2 = subtree:add(f.mydata, MpegData,"MPEG-1 Data")
    else
        subtree2 = subtree:add(f.mydata, MpegData,"Ram unknown format Data")
    end

    
    pinfo.cols.info = infotxt
end

-- load the udp.port table
tcp_table = DissectorTable.get("tcp.port")
-- register our protocol to handle tcp port 8000
tcp_table:add(8000,adts_proto)
udp_table = DissectorTable.get("udp.port")
-- register our protocol to handle udp port 8000
udp_table:add(8000,adts_proto)


